package org.bdware.mockjava;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import org.bdware.mockjava.generator.Generator;

import java.util.HashMap;
import java.util.Map;

public class MockGenerator extends JsonVisitor {

    public static MockGenerator instance = new MockGenerator();
    Map<String,ValueGenerator> prefixToGenerator;
    JsonElement result;
    public JsonElement get() {
        return result;
    }
    public  MockGenerator(Class ...clzs){
        prefixToGenerator = new HashMap<>();
        for (Class clz:clzs){
            Generator generator =(Generator) clz.getAnnotation(Generator.class);
            if (generator!=null){
                try {
                    prefixToGenerator.put(generator.tag(),(ValueGenerator)clz.getConstructor().newInstance());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }
    @Override
    public JsonVisitor visitObject(JsonObject jsonObject) {
        JsonObject jo = new JsonObject();
        for (String key : jsonObject.keySet()) {
            JsonElement origin = jsonObject.get(key);
            visit(origin);
            JsonElement je = get();
            if (origin.isJsonArray()) {
                key += "|" + Math.min(((JsonArray) origin).size(), 20);
            }
            jo.add(key, je);
        }
        result = jo;
        return this;
    }

    @Override
    public JsonVisitor visitJsonArray(JsonArray array) {
        JsonArray ret = new JsonArray();
        if (array.size() > 0) {
            visit(array.get(0));
            ret.add(result);
        }
        result = ret;
        return this;
    }

    @Override
    public JsonVisitor visitPrimitive(JsonPrimitive primitive) {
        if (!primitive.isString()) result = primitive.deepCopy();
        ValueGenerator generator = lookupGenerator(primitive.getAsString());
        result = generator.generate( primitive);
        return this;
    }

    private ValueGenerator lookupGenerator(final String str) {
        if (!str.startsWith("@")) {
            return new ValueGenerator() {
                @Override
                public JsonElement generate(  JsonElement previous) {
                    return new JsonPrimitive(str);
                }
            };
        }
        int from = str.indexOf("(");
        String prefix = from>=0?str.substring(0,from):str;
        ValueGenerator generator  = prefixToGenerator.get(prefix);
        if (generator==null){
            return new ValueGenerator() {
                @Override
                public JsonElement generate(JsonElement previous) {
                    return new JsonPrimitive("unknown type, prefix:"+prefix+" all:"+str);
                }
            };
        }
        return generator;
    }
}
