package org.bdware.mockjava;

import com.google.gson.*;
import jdk.nashorn.api.scripting.ScriptObjectMirror;
import jdk.nashorn.internal.runtime.Context;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

public class MockUtil {
    private static final String MOCK_JS_PATH = "org/bdware/mockjava/mock.js";
    public static ScriptEngine MOCK_JS_ENGINE;

    static {
        MOCK_JS_ENGINE = new ScriptEngineManager().getEngineByName("JavaScript");
        // System.out.println(MOCK_JS_ENGINE);
        try (InputStream mockJs =
                     MockUtil.class.getClassLoader().getResourceAsStream(MOCK_JS_PATH);
             InputStreamReader reader = new InputStreamReader(mockJs)) {
            MOCK_JS_ENGINE.eval(reader);
        } catch (ScriptException | IOException e) {
            // log.error("执行MockJs错误", e);
            System.out.println("执行MockJs错误1");
            System.out.println(e);
        }
    }

    private static boolean isJson(String str) {
        try {
            JsonParser.parseString(str).getAsJsonObject();
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    public static Object mock(String template) {
        Object result = ScriptObjectMirror.wrap(null, Context.getGlobal());
        if (template.length() > 0) {
            try {
                //
                // InputStreamReader reader = new InputStreamReader();
                // System.out.println(MOCK_JS_ENGINE.getContext());
                result = MOCK_JS_ENGINE.eval("JSON.stringify(Mock.mock(" + template + "))");
                System.out.println(result);
                // result = MOCK_JS_ENGINE.eval("JSON.stringify(Mock.mock())").toString();
            } catch (ScriptException e) {
                // log.error("执行Mock.mock错误", e);
                e.printStackTrace();
                System.out.println("执行MockJs错误2");
            }
        }
        return result;
    }

    public static JsonElement parseJsonSchema(JsonElement ele) {
        if (ele == null) return null;
        if (ele.isJsonPrimitive()) {
            JsonPrimitive primitive = ele.getAsJsonPrimitive();
            if (primitive.isBoolean()) {
                return new JsonPrimitive("@boolean");
            } else if (primitive.isNumber()) {
                return new JsonPrimitive("@integer");
            }
            return new JsonPrimitive("@string");
            // TODO
        } else if (ele.isJsonArray()) {
            JsonArray array = (JsonArray) ele;
            JsonArray ret = new JsonArray();
            if (array.size() > 0) {
                ret.add(parseJsonSchema(array.get(0)));
            }
            return ret;
            // [{"score":1},,,,] --> [{"score":"@integer"}]
        } else if (ele.isJsonObject()) {
            JsonObject jo = new JsonObject();
            JsonObject eleObj = ele.getAsJsonObject();
            for (String key : eleObj.keySet()) {
                JsonElement origin = eleObj.get(key);
                JsonElement je = parseJsonSchema(origin);
                if (origin.isJsonArray()) {
                    key += "|" + Math.min(((JsonArray) origin).size(), 20);
                }
                jo.add(key, je);
            }

            return jo;
        } // else if(ele.)
        return new JsonPrimitive("@string");
    }


}
